/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

//package hypertree;

import java.awt.Graphics;
import java.awt.FontMetrics;
import java.util.*;


/**
 * The HTDrawNode class contains the drawing coordinates of a HTModelNode 
 * for the HTView. 
 * It implements the Composite design pattern.
 */
class HTDrawNode {

    private   HTDraw              model    = null;  // drawing model
    private   HTModelNode         node     = null;  // encapsulated HTModelNode
 
    private   HTCoordE            ze       = null;  // current euclidian coordinates
    private   HTCoordE            oldZe    = null;  // old euclidian coordinates
    protected HTCoordS            zs       = null;  // cu rrent screen coordinates

    private   HTDrawNodeComposite father   = null;  // father of this node
    private   HTDrawNode          brother  = null;  // brother of this node

    private   HTNodeLabel         label    = null;  // label of the node

    protected boolean             fastMode = false; // fast mode


  /* --- Constructor --- */

    /**
     * Constructor.
     *
     * @param father    the father of this node
     * @param node      the encapsulated HTModelNode
     * @param model     the drawing model
     */
    HTDrawNode(HTDrawNodeComposite father, HTModelNode node, HTDraw model) {
        this.father = father;
        this.node = node;
        this.model = model;

        label = new HTNodeLabel(this);

        ze = new HTCoordE(node.getCoordinates());
        oldZe = new HTCoordE(ze);
        zs = new HTCoordS();
    }


  /* --- Brother --- */
  
    /**
     * Sets the brother of this node.
     *
     * @param brother    the borther of this node
     */

    void setBrother(HTDrawNode brother) {
        this.brother = brother;
    }


    /**
     * Returns the border type of the node
     *
     * @return      String
     *              
     */

    String getBorderType() 
    {
		return model.getBorderType();
    }

    /**
     * Returns the hint color via model class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getHintColor() 
    {
		return model.getHintColor();
    }

    /**
     * Returns the name of the font via model class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getFontNode() {
		return model.getFontNode();
    }

    /**
     * Returns the size of the font, via model class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    int getFontSizeNode() {
		return model.getFontSizeNode();
    }

    /**
     * Returns the search color via model class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getSearchColor() 
    {
		return model.getSearchColor();
    }

    /**
     * Returns the link color via model class
     * contains thoses given in parameters.
     *
     * @return      String of RGB color
     *              
     */

    String getLinkColor() 
    {
		return model.getLinkColor();
    }

    /**
     * Returns the text of search box, if has one.
     *
     *
     * @return      String 
     *              
     */

    String getSearchWord() {
		return (model.getSearchWord());
    }

    String getHelpFile() {
		return (model.getHelpFile());
    }

    String getUrlAdr() {
    		return (model.getUrlAdr());
    }



    /**
     * Returns the children of this node in an Enumeration.
     * If this node is a file, return a empty Enumeration.
     * Else, return an Enumeration full with HTFileNode.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration childrenNode() {
        return node.childrenNode();
    }

  /* --- Name --- */

    /**
     * Returns the name of this node.
     *
     * @return    the name of this node
     */
    String getName() {
        return node.getName();
    }

    /**
     * Returns the color of this node.
     *
     * @return    the color of this node
     */

    String getColor() {
        return node.getColor();
    }

    /**
     * Returns the TextColor of this node.
     *
     * @return    the color of this node
     */

    String getTextColor() {
        return node.getTextColor();
    }


    /**
     * Returns the URL of this node.
     *
     * @return    the URL of this node
     */

    String getURL() {
        return node.getURL();
    }

    /**
     * Returns the ID of this node.
     *
     * @return  ID  of this node
     */

    String getID() {
        return node.getID();
    }

   /**
     * get Highlight of the node.
     *
     * 
     */

    boolean getEspecialMark() {
        return (node.getEspecialMark());
    }

    /**
     * Returns the attributes hint of the node.
     * @return the hint of the node 
     */
    public String getHint() {
	  return node.getHint();
    }

    /**
     * Returns the attributes hasHint of the node.
     * @return boolean 
     */
    public boolean getHasHint() {
	  return node.getHasHint();
    }

   public boolean isLeafNode() {
        return (node.isLeafNode());
    }


    /**
     * Returns the attributes hasData of the node.
     * @return boolean 
     */
    public boolean getHasData() {
	  return node.getHasData();
    }

    /**
     * Set the attributes hint of the node.
     * @void
     */
    public void setHasHint(boolean v) {
	  node.setHasHint(v) ;
    }

    /**
     * Set the attributes hasData of the node.
     * @void
     */
    public void setHasData(boolean v) {
	  node.setHasData(v) ;
    }

    /**
     * set true or false Highlight of the node.
     *
     * 
     */

    void setEspecialMark(boolean t) {
        node.setEspecialMark(t);
    }


    // 13/09/2004

    String getNodeClientData() {
	return(node.getNodeClientData());
    }

    public void setNodeClientData(String v) {
	  node.setNodeClientData(v);
    }

    // END



    /**
     * set true or false hasArc of the node.
     *
     * 
     */

    void setHasArc(boolean t) {
        node.setHasArc(t);
    }

    /**
     * get hasArc of the node.
     *
     * 
     */

    boolean getHasArc() {
        return (node.getHasArc());
    }


    /**
     * Returns the HTNodeLabel of this node.
     *
     * @return  HTNodeLabel of this node
     */

    HTNodeLabel getHTNodeLabel() {
        return label;
    }

    HTDrawNode getBrother()
    {
		return brother;
    }

    /**
     * Returns the width of the Panel.
     *
     * @return  int
     */

    int getWidthPanel() {
        return model.getWidthPanel();
    }

    /**
     * Returns the length of this node.
     *
     * @return  int
     */

    int getHeighthPanel() {
        return model.getHeightPanel();
    }

    int getAscent() {
		return(model.getAscent());
    }

    int getLeading() {
		return (model.getLeading());
    }

    int getCharWidth() {
		return(model.getCharWidth());
    }

    int getCharHeight() {
		return (model.getCharHeight());
    }

    int  getStringWidth( char[] nameC, int init, int end) {
		return (model.getStringWidth(nameC, init, end));
    }            

    int  getStringWidth( String nameC) {
		return (model.getStringWidth(nameC));
    }            


    /* --- Coordinates --- */

    /**
     * Returns the current coordinates of this node.
     * WARNING : this is NOT a copy but the true object
     * (for performance).
     *
     * @return     the current coordinates
     */
    HTCoordE getCoordinates() {
        return ze;
    } 

    HTCoordE getOldCoordinates() {
        return oldZe;
    }

    HTCoordS getScreenCoordinates() {
        return zs;
    }

    /**
     * Refresh the screen coordinates of this node.
     *
     * @param sOrigin   the origin of the screen plane
     * @param sMax      the (xMax, yMax) point in the screen plane
     */
    void refreshScreenCoordinates(HTCoordS sOrigin, HTCoordS sMax) {
        zs.projectionEtoS(ze, sOrigin, sMax);
    } 


  /* --- Drawing --- */


    /**
     * Draws the branches from this node to 
     * its children.
     * Overidden by HTDrawNodeComposite
     *
     * @param g    the graphic context
     */
    void drawBranches(Graphics g) {}

    /**
     * Draws this node.
     *
     * @param g    the graphic context
     */
    void drawNodes(Graphics g) {
        //if (fastMode == false) {
            label.draw(g, fastMode);

        //}
    }

    /**
     * Draws hint of the node.
     *
     * @param g    the graphic context
     */
    void drawHint(Graphics g) {
        //if (fastMode == false) {
            label.drawHint(g);
        //}
    }


    /**
     * Returns the minimal distance between this node
     * and his father and his brother.
     *
     * @return    the minimal distance
     */
    int getSpace() {
        int dF = -1;
        int dB = -1;
        int dxP = -1;
	  int dyB = -1;
	  boolean emColuna = false;
	  HTCoordS zF = null;
	  HTCoordS zB = null;
	         
        if (father != null) {
            zF = father.getScreenCoordinates();
            dF  = zs.getDistance(zF);
		dxP = zs.getDistanceX(zF);
        }
        if (brother != null) {
          	zB = brother.getScreenCoordinates();
        	dB = zs.getDistance(zB);
		dyB = zs.getDistanceY(zB);	
        }
      
        if ((dF == -1) && (dB == -1)) {
            return -1;
        } else if (dF == -1) {
            return dB;
        } else if (dB == -1) {
            return dF;
        } else { 
		int min = Math.min(dF, dB);

		if (min <= model.getCharHeight()+1)
			return min;
		else {
			if ((dxP > dyB) && (dyB > model.getCharHeight()+1)) {
				if ( (Math.abs(zs.x-zF.x) < dF/2) || ((Math.abs(zs.x-zB.x) < dF) && 
                                                              (Math.abs(zs.y-zB.y) < model.getCharHeight()) ) )				
					return min;
				else if (isLeafNode())
					return ((dF+dB)/2);				
				else
					return (dF);
			}
			else
				return min;		
		}
        }
	  
    }

  /* --- Translation --- */

    /**
     * Translates this node by the given vector.
     *
     * @param t    the translation vector
     */
    void translate(HTCoordE t) {
        ze.translate(oldZe, t);
    }

    /**
     * Transform this node by the given transformation.
     *
     * @param t    the transformation
     */
    void transform(HTTransformation t) {
        ze.copy(oldZe);
        ze.transform(t);
    }

    /**
     * Ends the translation.
     */
    void endTranslation() {
        oldZe.copy(ze);
    }

    /**
     * Restores the hyperbolic tree to its origin.
     */
    void restore() {
        HTCoordE orig = node.getCoordinates();
        ze.x = orig.x;
        ze.y = orig.y;
        oldZe.copy(ze);
    }

    /**
     * Sets the fast mode, where nodes are no more drawed.
     *
     * @param mode    setting on or off.
     */
    void fastMode(boolean mode) {
        if (mode != fastMode) {
            fastMode = mode;
        }
    }


  /* --- Node searching --- */

    /**
     * Returns the node (if any) whose screen coordinates' zone
     * contains thoses given in parameters.
     *
     * @param zs    the given screen coordinate
     * @return      the searched HTDrawNode if found;
     *              <CODE>null</CODE> otherwise
     */
    HTDrawNode findNode(HTCoordS zs) {
        if (label.contains(zs)) {
            return this;
        } else {
            return null;
        }
    }


    HTDrawNode findNode(String  idNode) 
    { 
	if (getID().equals(idNode)) {
            setEspecialMark(true);
		return this;
      }
	else 
		return null;	
    }

    // 13/09/2004

    int verifyFolder(HTCoordS zs) {
        return(label.verifyFolder(zs));
    }

    // END


  /* --- ToString --- */

    /**
     * Returns a string representation of the object.
     *
     * @return    a String representation of the object
     */
    public String toString() {
        String result = getName() + 
                        "\n\t" + ze + 
                        "\n\t" + zs; 
        return result;
    }

    public void insereFilho()
    {		
		node.insereFilho();

    }
    public void deleteFilho()
    {		
		node.deleteFilho();

    }
    
    public void setHintPai(boolean v) {

	    	node.setHintPai(v);
    }    
    
    public void  setInseredChild(boolean v) {
	    	node.setInseredChild(v);    
    }
    
    public HTNode findRamo(String id) {
		return(node.findRamo(id));
    }

   public void insereRamo(String id)
    {		
		node.insereRamo(id);

    }

   public void insereEspelho(String id)
    {		
		node.insereEspelho(id);

    }

   public void insereHideLink(String id)
    {		
		node.insereHideLink(id);

    }



   public String editFilho()
   {		
		return(node.editFilho());

   }

   public void saveFilho(String texto)
    {		
		node.saveFilho(texto);

    }


    public void aplicarCoresFilhos(String cores)
    {		
		node.aplicarCoresFilhos(cores);

    }

    public HTModelNode getHTModelNode() {		
		return(node);	
    }

    public boolean getHasHintPai() {		
		return(node.getHasHintPai());	
    }
    int getNumAnds() {
	return(model.getNumAnds());
    }
  
    String getWordI(int i) {
	return (model.getWordI(i));
    }
   

}

